#ifndef PSRMETADATA_HH_
#define PSRMETADATA_HH_
#include <fitsio.h>
#include <string>
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

namespace PVO
{

  namespace Meta
  {

    /** The precision for printing floats */
    const int     fltwidth=4;

    std::string   floatToString(float f);
    std::string   intToString(int i);

    /**
     * The namespace to record only relevant details about a PSRFITS
     * file, that can be used by the utility programs pvosummary and
     * pvoconesearch.
     */
    class Metadata
    {
    public:
      Metadata(){sepFlag=false; angsep=nullFloat;};
      virtual ~Metadata(){};

      void        define(FITS::PSRFits fitsfile);
      friend bool operator< (Metadata lhs, Metadata rhs){
	return (lhs.angsep < rhs.angsep);
      };

      void        setAngSep(float ra, float dec){angsep = angSep(ra, dec, rajd, decjd); sepFlag=true;};
      bool        close(float radius);
      bool        close(float ra, float dec, float radius);

      std::string getFilename(){return filename;};
      void        setFilename(std::string s){filename=s;};
      std::string getFileloc(){return fileloc;};
      void        setFileloc(std::string s){fileloc=s;};
      std::string getHeaderVersion(){return headerversion;};
      void        setHeaderVersion(std::string s){headerversion=s;};
      std::string getSrcname(){return srcname;};
      void        setSrcname(std::string s){srcname=s;};
      std::string getRaS(){return raj;};
      void        setRaS(std::string s){raj=s;};
      std::string getDecS(){return decj;};
      void        setDecS(std::string s){decj=s;};
      float       getRaD(){return rajd;};
      void        setRaD(float f){rajd=f;};
      float       getDecD(){return decjd;};
      void        setDecD(float f){decjd=f;};
      std::string getDate(){return date;};
      void        setDate(std::string s){date=s;};
      float       getBeamsize(){return beamsize;};
      void        setBeamsize(float f){beamsize=f;};
      float       getBmaj(){return bmaj;};
      void        setBmaj(float f){bmaj=f;};
      float       getBmin(){return bmin;};
      void        setBmin(float f){bmin=f;};
      float       getBpa(){return bpa;};
      void        setBpa(float f){bpa=f;};
      float       getObsfreq(){return obsfreq;};
      void        setObsfreq(float f){obsfreq=f;};
      float       getTbin(){return tbin;};
      void        setTbin(float f){tbin=f;};
      float       getScanlen(){return scanlen;};
      void        setScanlen(float f){scanlen=f;};
      int         getNchan(){return nchan;};
      void        setNchan(int i){nchan=i;};
      float       getChanbw(){return chanbw;};
      void        setChanbw(float f){chanbw=f;};
      float       getAngSep(){if(sepFlag) return angsep; else return -1.;};
      void        setAngSep(float f){angsep=f; sepFlag=true;};
      bool        showSep(){return sepFlag;};

      std::string getBeamsizeS();
      std::string getObsfreqS();
      std::string getTbinS();
      std::string getAngSepS();

    private:
      std::string filename;
      std::string fileloc;
      std::string headerversion;
      std::string srcname;
      std::string raj;
      std::string decj;
      float       rajd;
      float       decjd;
      std::string date;
      float       beamsize;
      float       bmaj;
      float       bmin;
      float       bpa;
      float       obsfreq;
      float       tbin;
      float       scanlen;
      int         nchan;
      float       chanbw;
      float       angsep;
      bool        sepFlag;
    };

    void printMetadataShort(std::ostream &theStream, Metadata data);
    void printMetadataShort(std::ostream &theStream, std::vector<Metadata> datalist);
    void printMetadataFull(std::string filename, Metadata data);
    void printMetadataFull(std::string filename, std::vector<Metadata> datalist);

  }
}

#endif //PSRSUMMARY_HH_
