#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"
#include "pvo-metadata.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using namespace PVO::Meta;
using std::string;


int main(int argc, char * argv[])
{
  /** 
   * PVOSummary is a function to print out basic information for
   * PSRFITS files.
   *
   * This program is a command-line program that looks at PSRFITS
   * files (i.e. files that have extensions .fits or .rf) and returns
   * a summary of their positions, beam sizes and observation times.
   *
   * The format of usage is: <br> 
   * <tt>> pvosummary </tt><br>
   * to analyse all PSRFITS files in the current directory, or <br>
   * <tt>> pvosummary FILE</tt><br>
   * to analyse the file FILE. Entering<br>
   * <tt>> pvosummary -h</tt><br>
   * will give a help message, using PVO::pvosummary_usage_text.
   *
   * The output is a list of valid files, their central RA & DEC,
   * their beam size, the central observing frequency, and the
   * DATE-OBS keyword (time/date of observation),
   * 
   */


  bool verboseflag = false;

  std::vector<std::string> namelist;
  unsigned int i;
  switch(argc){
  case 1:
  case 3:
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];
    if(argc==3){
    }
    break;
  case 2:
  case 4:
    if(strcmp(argv[1],"-h")==0){
      std::cout << pvosummary_usage_text;
      return 0;
    }
    namelist.resize(1);
    namelist[0] = argv[1];
    if(argc==4){
    }
    break;
  default:
    std::cerr << pvosummary_usage_text;
    return 0;
    break;
  }


  std::vector<Metadata> goodlist;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      Metadata newmatch;
      newmatch.define(fitsfile);
      goodlist.push_back(newmatch);
    }
  }

  printMetadataShort(std::cout,goodlist);

  printMetadataFull("PVOsummary.csv",goodlist);

}
